/*
无边星链合成兑换
*/


const scriptName = `无边星链合成兑换`;
const magicJS = MagicJS(scriptName, "INFO");

const WuBianConstKey = {
    // 基础数据
    Token: 'WubianProToken',
    ActivityListData: 'WubianActivityListData',
    ActivityGoodsList: 'WubianActivityGoodsList',
    ActivityTaskCaptureEnabled: 'WubianActivityTaskCaptureEnabled',
    ActivityTagName: 'WubianActivityActivityTagName',
    ActivityIndexName: 'WubianActivityIndexName',
    ActivityId: 'WubianActivityId',
    SynthesisConcurrentMode: 'WubianSynthesisConcurrentMode',
    SynthesisRunsPerSecond: 'WubianSynthesisRunsPerSecond',
    SynthesisAmount: 'WubianSynthesisTargetAmount',

    SynthesisCompositeGoodsListReqTime: 'WubianSynthesisCompositeGoodsListReqTime',
    SynthesisConvertGoodsListReqTime: 'WubianSynthesisConvertGoodsListReqTime',
    SynthesisActivityDisabled: 'WubianSynthesisActivityDisabled',
    SynthesisAllowAdvanceTime: 'WubianSynthesisAllowAdvanceTime',
    SynthesisUseCacheEnabled: 'WubianSynthesisUseCacheEnabled',
};

const gUserAgent = `Mozilla/5.0 (iPhone; CPU iPhone OS 16_6_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Html5Plus/1.0 (Immersed/20) uni-app`;
const gHost = `api.wubian.pro`;
let gToken = magicJS.data.read(WuBianConstKey.Token, '');
const gCommonHeaders = {
    'Accept': `*/*`,
    'Accept-Encoding': `gzip, deflate, br`,
    'Connection': `keep-alive`,
    'Content-Type': `application/json`,
    'Cookie': `token=${gToken}`,
    'Host': gHost,
    'User-Agent': gUserAgent,
    'CLIENT-TYPE': `APP`,
    'token': gToken,
    'Accept-Language': `zh-CN,zh-Hans;q=0.9`
};

let gRetBody;

let gSynthesizeResultList = [];
let gSynthesizeResultDict = {};

async function Main() {
    if (magicJS.isStrictRequest) {
        magicJS.checkRecordRequestBody();
    }
    if (magicJS.isRequest) {
        checkHandleRequest();
    } else {
        updateHeaders();
        await trySynthesize();
    }
    magicJS.notification.msg('');
    if (gRetBody) {
        magicJS.done({
            body: JSON.stringify(gRetBody)
        });
    } else {
        magicJS.done();
    }
}



function checkHandleRequest() {
}

function handleHeaders() {
    if ($request && $request.method != 'OPTIONS') {
        let headers = $request.headers;
        let newToken = headers['token'];
        let oldToken = magicJS.data.read(WuBianConstKey.Token, null);
        if (newToken) {
            if (newToken != oldToken) {
                magicJS.logger.info(`当前Token刷新到最新`);
                magicJS.notification.appendNotifyInfo(`🎉当前Token刷新到最新`);
            } else {
                magicJS.logger.info(`当前Token已经是最新无需更新`);
            }
            magicJS.data.write(WuBianConstKey.Token, newToken);
        } else {
            let newCookieStr = headers['Cookie'];
            if (newCookieStr) {
                let cookieDict = magicJS.parseCookies(newCookieStr);
                if (cookieDict['token'] != void 0) {
                    newToken = ookieDict['token'];
                    magicJS.data.write(WuBianConstKey.Token, newToken);
                    magicJS.logger.info(`当前Token刷新到最新`);
                }
            }
        }
    }
}

function updateHeaders() {
    gToken = magicJS.data.read(WuBianConstKey.Token, '');
    gCommonHeaders['token'] = gToken;
    gCommonHeaders['cookie'] = `token=${gToken}`;
}

function getResponsePlainData() {
    let data = magicJS.getResponseBody();
    if (!data) {
        return;
    }
    try {
        return JSON.parse(data);
    } catch (err) {
        magicJS.logger.error(err);
    }
}

function getRequestPlainData() {
    let data = magicJS.getRequestBody();
    let reqData = null;
    if (data) {
        reqData = JSON.parse(data);
    } else {
        reqData = magicJS.parseQueryStr($request.url);
    }
    return reqData;
}

function getQueryPlainData() {
    let params = magicJS.parseQueryStr($request.url);
    return params;
}

function compareCookieStr(cookieStr1, cookieStr2) {
    let cookieDict1 = magicJS.parseCookies(cookieStr1);
    let cookieDict2 = magicJS.parseCookies(cookieStr2);
    let skipKeys = [];
    let retCode = 0;
    for (let key in cookieDict1) {
        if (!skipKeys.includes(key)) {
            if (cookieDict1[key] != void 0 && cookieDict2[key] != void 0 && cookieDict1[key] != cookieDict2[key]) {
                magicJS.logger.info(`${key}`);
                retCode = -1;
                break;
            }
        }
    }
    return retCode;
}

function appendCookieStr(newCookieStr) {
    let cookieStr = magicJS.data.read(WuBianConstKey.Cookie, null);
    let cookieDict = magicJS.parseCookies(cookieStr);
    let newCookieDict = magicJS.parseCookies(newCookieStr);
    for (let key in newCookieDict) {
        cookieDict[key] = newCookieDict[key];
    }
    cookieStr = magicJS.serializeCookies(cookieDict);
    magicJS.data.write(WuBianConstKey.Cookie, cookieStr);
}

function handleUserCollectList() {
    let rspData = getResponsePlainData();
    if (!rspData) {
        return;
    }
    if (rspData.code != 200) {
        return;
    }
    handleHeaders();
}

function handleInitData() {
    let rspData = getResponsePlainData();
    if (!rspData) {
        return;
    }
    if (rspData.code != 200) {
        return;
    }
    handleHeaders();
}

function getWillActivityTagName() {
    let tagName = magicJS.data.read(WuBianConstKey.ActivityTagName, '');
    return tagName;
}

function getWillActivityIndexName() {
    let indexName = magicJS.data.read(WuBianConstKey.ActivityIndexName, '');
    return indexName;
}

function getWillActivityId() {
    let activityId = magicJS.data.read(WuBianConstKey.ActivityId, '');
    return activityId;
}


function getActivityWillCount() {
    return 99999;
}

function isConcurrentMode() {
    let flag = magicJS.data.read(WuBianConstKey.SynthesisConcurrentMode, true);
    return flag;
}

function getRunsPreSecond() {
    let runs = magicJS.data.read(WuBianConstKey.SynthesisRunsPerSecond, 5);
    return runs;
}

async function getWillActivityData() {
    let tagName = getWillActivityTagName();
    let indexName = getWillActivityIndexName();
    let activityId = getWillActivityId();
    let activityType = getActivityType(tagName);
    let activityLstMap = await getActivityList(activityType);
    let activityData = null;
    magicJS.logger.info(`activityType=${activityType} activityId=${activityId}`);
    // magicJS.logger.info(`${JSON.stringify(activityLstMap)}`);
    for (let hashId in activityLstMap) {
        let item = activityLstMap[hashId];
        if ((item.activityId == activityId || item.hashId == activityId) || (indexName.length > 0 && item.name.indexOf(indexName) != -1)) {
            activityData = item;
        }
    }
    if (!activityData) {
        let key = `${WuBianConstKey.ActivityGoodsList}#${activityId}`;
        let retCvtData = magicJS.data.read(key, null);
        if (retCvtData) {
            activityData = retCvtData.activityInfo;
        }
    }
    return activityData;
}

function setCompositeGoodsListReqTime(time) {
    magicJS.data.write(WuBianConstKey.SynthesisCompositeGoodsListReqTime, time);
}

function setConvertGoodsListReqTime(time) {
    magicJS.data.write(WuBianConstKey.SynthesisConvertGoodsListReqTime, time);
}

function checkCompositeGoodsListReqForceNeed() {
    let time = magicJS.data.read(WuBianConstKey.SynthesisCompositeGoodsListReqTime, 0);
    let diffTime = Date.now() - Number(time);
    return diffTime > 10 * 1000;
}

function checkConvertGoodsListReqForceNeed() {
    let time = magicJS.data.read(WuBianConstKey.SynthesisConvertGoodsListReqTime, 0);
    let diffTime = Date.now() - Number(time);
    return diffTime > 10 * 1000;
}

function checkActivityDisable(activityId) {
    let key = `${WuBianConstKey.SynthesisActivityDisabled}#${activityId}`;
    let bDisable = magicJS.data.read(key, false);
    return bDisable;
}

function setActivityDisabled(activityId, bDisabled = true) {
    let key = `${WuBianConstKey.SynthesisActivityDisabled}#${activityId}`;
    magicJS.data.write(key, bDisabled);
}

function getSynthesisAmount() {
    let amount = magicJS.data.read(WuBianConstKey.SynthesisAmount, 10);
    return Number(amount);
}

function setSynthesisAmount(amount) {
    magicJS.data.write(WuBianConstKey.SynthesisAmount, amount);
}

function checkActivityAutoDisable(activityId) {
    for (let retData of gSynthesizeResultList) {
        if (retData && retData.code == 500) {
            if (retData.msg && (retData.msg.indexOf('库存不足') != -1 || retData.msg.indexOf('次数不足') != -1)) {
                setActivityDisabled(activityId, true);
                return true;
            }
        }
    }
    return false;
}

function getAllowAdvanceTime() {
    return magicJS.data.read(WuBianConstKey.SynthesisAllowAdvanceTime, 5);
}

function isSynthesisUseCacheEnabled() {
    return magicJS.data.read(WuBianConstKey.SynthesisUseCacheEnabled, false);
}

function checkIsSynthesizeTaskSkip(index) {
    let data = gSynthesizeResultDict[index];
    if (data && data.result) {
        let result = data.result;
        if (result.code == 200) {
            magicJS.logger.info(`序号[${index + 1}]任务已经执行成功过，无需再次执行[0]`);
            return true;
        }
        if (result.code == 500 && (result.msg.indexOf('单品状态发生变化') != -1)) {
            magicJS.logger.info(`序号[${index + 1}]任务已经执行成功过，无需再次执行[1]`);
            return true;
        }
        magicJS.logger.info(`序号[${index + 1}]任务上次执行发送异常，错误信息：${result.msg}`);
    }
    return false;
}

function checkUpdateActivityDataCache(activityId) {
    let key = `${WuBianConstKey.ActivityGoodsList}#${activityId}`;
    let activityData = magicJS.data.read(key, null);
    if (activityData) {
        let goodsListData = activityData.goodsList;
        let needUpdate = true;
        for (let index in gSynthesizeResultDict) {
            let detail = gSynthesizeResultDict[index];
            let result = detail.result;
            if (result) {
                if (result.code == 200 || (result.code == 500 && (result.msg.indexOf('单品状态发生变化') != -1))) {
                    let costGoodsList = detail.goodsList;
                    let costGoodsDict = {};
                    for (let goodsId of costGoodsList) {
                        costGoodsDict[goodsId] = 1;
                    }
                    magicJS.logger.info(`costGoodsDict=${JSON.stringify(costGoodsDict)}`);
                    magicJS.logger.info(`goodsListData=${JSON.stringify(goodsListData)}`);
                    for (let i = 0; i < goodsListData.length; i++) {
                        checkRemoveVerList(goodsListData[i].verList, costGoodsDict);
                    }
                    needUpdate = true;
                }
            }
        }
        if (needUpdate) {
            magicJS.data.write(key, activityData);
            let activityInfo = activityData.activityInfo;
            magicJS.logger.info(`更新活动[${activityInfo.name}]任务数据缓存成功`);
        }
    }
}

function checkRemoveVerList(srcVerList, removeData) {
    for (let i = srcVerList.length - 1; i >= 0; i--) {
        const verInfo = srcVerList[i];
        if (removeData[verInfo.goodsId]) {
            srcVerList.splice(i, 1);
        }
    }
    return srcVerList;
}

async function trySynthesize() {
    magicJS.logger.info(`[1]活动任务执行前检查...`);
    let activityData = await getWillActivityData();
    // magicJS.logger.info(`activityData=${JSON.stringify(activityData)}`);
    if (!activityData) {
        return;
    }
    let nowTime = Date.now() / 1000;
    let diffTimeSec = getAllowAdvanceTime();
    if (nowTime < activityData.startTime - diffTimeSec) {
        magicJS.notification.appendNotifyInfo(`⚠️活动[${activityData.name}]未开始，距离开始时间${Math.floor(activityData.startTime - nowTime)}秒，请在活动开始前${diffTimeSec}秒内执行！`);
        return;
    }
    magicJS.logger.info(`[2]活动任务执行前检查...`);
    let activityId = activityData.hashId || activityData.activityId;
    let retCvtData = null;
    if (isSynthesisUseCacheEnabled()) {
        magicJS.logger.info(`当前使用的活动数据来自缓存，如不需在设置中关闭使用缓存功能`);
        let key = `${WuBianConstKey.ActivityGoodsList}#${activityId}`;
        retCvtData = magicJS.data.read(key, null);
    }
    if (!retCvtData) {
        if (activityData.type == 0) {
            retCvtData = await doGetCompositeGoodsList(activityId, checkCompositeGoodsListReqForceNeed());
        } else if (activityData.type == 1) {
            retCvtData = await doGetConvertGoodsList(activityId, checkConvertGoodsListReqForceNeed());
        }
    }
    magicJS.logger.info(`[3]活动任务执行前检查...`);
    // magicJS.logger.info(`retCvtData=${JSON.stringify(retCvtData)}`);
    if (!retCvtData) {
        magicJS.notification.appendNotifyInfo(`⚠️活动[${activityData.name}]详细合成信息数据不存在`);
        return;
    }
    if (checkActivityDisable(activityId)) {
        magicJS.notification.appendNotifyInfo(`⚠️活动[${activityData.name}]已禁用，请重新配置`);
        return;
    }
    let willCount = getActivityWillCount();
    willCount = getSynthesisAmount();
    let combinationLst = getSynthesizeCombination(retCvtData);
    // magicJS.logger.info(`combinationLst=${JSON.stringify(combinationLst)}`);
    let nExcuteCount = Math.min(willCount, combinationLst.length);
    let tipsText = `[${activityData.activityType}]${activityData.name}，执行结果如下：\n`;

    let nRuns = getRunsPreSecond();
    let ctrlMillSec = 1000 / nRuns;
    let isConcurrentExcute = isConcurrentMode();
    if (nExcuteCount == 0) {
        tipsText += `由于缺少可执行的材料组合，该活动自动禁用\n`;
        tipsText += `请检查活动重新配置，或购买相应材料，重新开启活动任务`;
        tipsText = `❌` + tipsText;
        setActivityDisabled(activityId, true);
    } else {
        if (isConcurrentExcute) {
            nowTime = Date.now() / 1000;
            let waittingMSec = Math.floor((activityData.startTime - nowTime) * 1000);
            if (waittingMSec > 0) {
                magicJS.logger.info(`等待${waittingMSec}毫秒后开始执行...`);
                await magicJS.utils.sleep(waittingMSec - 80);
            }
            magicJS.logger.info(`活动任务开始执行...`);
            for (let n = 0; n < nRuns; n++) {
                let startTime = Date.now();
                for (let i = 0; i < nExcuteCount; i++) {
                    let oneCombination = combinationLst[i];
                    let goodsList = getCombinationGoodsList(oneCombination);
                    magicJS.logger.info(`第${n}-${i}趟开始执行...`);
                    if (checkIsSynthesizeTaskSkip(i)) {
                        magicJS.logger.info(`第${n}-${i}趟跳过执行...`);
                        continue;
                    }
                    if (activityData.type == 0) {
                        doCreateCompositeOrder(i, activityId, goodsList);
                    } else if (activityData.type == 1) {
                        doCreateConvertOrder(i, activityId, goodsList);
                    }
                }
                let costTime = Date.now() - startTime;
                if (costTime < ctrlMillSec) {
                    await magicJS.utils.sleep(ctrlMillSec - costTime);
                }
                if (checkActivityAutoDisable()) {
                    break;
                }
            }
            let nSuccCount = 0;
            let nFailCount = 0;
            for (let retData of gSynthesizeResultList) {
                if (retData && retData.code == 200) {
                    nSuccCount += 1;
                } else {
                    if (retData.code == 500 && retData.msg && retData.msg.indexOf('库存不足') != -1) {
                        setActivityDisabled(activityId, true);
                    }
                    nFailCount += 1;
                }
            }
            tipsText += `成功兑换次数：${nSuccCount}\n`;
            tipsText += `失败兑换次数：${nFailCount}\n`;
            tipsText += `未知结果次数：${nExcuteCount * nRuns - nSuccCount - nFailCount}\n`;
            if (nSuccCount >= willCount) {
                tipsText += `由于兑换次数已达到配置的次数，该活动自动禁用\n`;
                setActivityDisabled(activityId, true);
            } else {
                setSynthesisAmount(willCount - nSuccCount);
                if (isSynthesisUseCacheEnabled()) {
                    checkUpdateActivityDataCache(activityId);
                }
            }
            if (nSuccCount > 0) {
                tipsText = `🎉` + tipsText;
            } else {
                tipsText = `❌` + tipsText;
            }
        } else {
            nowTime = Date.now() / 1000;
            let waittingMSec = Math.floor((activityData.startTime - nowTime) * 1000);
            if (waittingMSec > 0) {
                magicJS.logger.info(`等待${waittingMSec}毫秒后开始执行...`);
                await magicJS.utils.sleep(waittingMSec - 80);
            }
            let nSuccCount = 0;
            let nFailCount = 0;
            for (let n = 0; n < nRuns; n++) {
                let startTime = Date.now();
                for (let i = 0; i < nExcuteCount; i++) {
                    let oneCombination = combinationLst[i];
                    let goodsList = getCombinationGoodsList(oneCombination);
                    let retData = null;
                    if (activityData.type == 0) {
                        retData = await doCreateCompositeOrder(i, activityId, goodsList);
                    } else if (activityData.type == 1) {
                        retData = await doCreateConvertOrder(i, activityId, goodsList);
                    }
                    let errMsg = '';
                    if (retData && retData.code == 200) {
                        nSuccCount += 1;
                        errMsg = `序号[${i + 1}]使用材料[${getCombinationGoodsLstText(oneCombination)}]兑换成功`;
                    } else {
                        nFailCount += 1;
                        errMsg = `序号[${i + 1}]使用材料[${getCombinationGoodsLstText(oneCombination)}]${retData?.msg || '兑换失败'}`;
                    }
                    tipsText += `${errMsg}\n`;
                }
                let costTime = Date.now() - startTime;
                if (costTime < ctrlMillSec) {
                    await magicJS.utils.sleep(ctrlMillSec - costTime);
                }
            }
            if (nSuccCount >= willCount) {
                tipsText += `由于兑换次数已达到配置的次数，该活动自动禁用\n`;
                setActivityDisabled(activityId, true);
            } else {
                setSynthesisAmount(willCount - nSuccCount);
            }
            if (nSuccCount > 0) {
                tipsText = `🎉` + tipsText;
            } else {
                tipsText = `❌` + tipsText;
            }
        }
    }
    magicJS.logger.info(`活动任务执行结束`);
    magicJS.notification.appendNotifyInfo(tipsText);
}

function getCombinationGoodsList(combination) {
    let retList = [];
    for (let item of combination) {
        retList.push(item.goodsId);
    }
    return retList;
}
function getCombinationGoodsLstText(combination) {
    let text = '';
    for (let item of combination) {
        text += `${item.title}#${item.ver},`;
    }
    return text.substring(0, text.length - 1);
}

function getSynthesizeCombination(retCvtData) {
    let goodsList = retCvtData.goodsList;
    let activityInfo = retCvtData.activityInfo;
    let ruleList = retCvtData.ruleList;
    if (!ruleList) {
        if (typeof activityInfo.limit == 'number') {
            if (activityInfo.articleId && activityInfo.articleId.length > 0) {
                ruleList = [{
                    num: activityInfo.limit,
                    desc: '',
                    matchArtIdList: [activityInfo.articleId],
                }];
            } else {
                ruleList = [{
                    num: activityInfo.limit,
                    desc: '',
                    matchArtIdList: [goodsList[0].artHashId],
                }];
            }
        }
    }
    let retAllList = [];
    let isContinus = true;
    while (isContinus) {
        let retOneList = [];
        // magicJS.logger.info(`ruleList${JSON.stringify(ruleList)}`);
        // magicJS.logger.info(`goodsList:${JSON.stringify(goodsList)}`);
        for (let i = 0; i < ruleList.length; i++) {
            let limitCount = ruleList[i].num;
            let matchArtIdList = ruleList[i].matchArtIdList;
            // magicJS.logger.info(`limitCount:${limitCount}`);
            // magicJS.logger.info(`matchArtIdList:${JSON.stringify(matchArtIdList)}`);
            let oneData = goodsList[i];
            let matchArtList = getSubGoodsList(goodsList, matchArtIdList, limitCount);
            if (matchArtList && matchArtList.length == limitCount) {
                // magicJS.logger.info(`matchArtList:${JSON.stringify(matchArtList)}`);
                for (let j = 0; j < matchArtList.length; j++) {
                    retOneList.push(matchArtList[j]);
                }
            } else {
                isContinus = false;
                break;
            }
        }
        if (isContinus) {
            retAllList.push(retOneList);
        }
    }
    return retAllList;
}

function getSubGoodsList(goodsList, matchArtIdList, num) {
    let retList = [];
    for (let i = 0; i < goodsList.length; i++) {
        const item = goodsList[i];
        if (matchArtIdList.includes(item.artHashId)) {
            let neddNum = num - retList.length;
            let diffNum = item.verList.length - neddNum;
            if (diffNum >= 0) {
                let tmpLst = item.verList.splice(item.verList.length - num, neddNum);
                // retList = retList.concat(tmpLst);
                for (let j = 0; j < tmpLst.length; j++) {
                    tmpLst[j].title = item.title;
                    retList.push(tmpLst[j]);
                }
                break;
            } else {
                let tmpLst = item.verList.splice(0, retList.length);
                // retList = retList.concat(tmpLst);
                for (let j = 0; j < tmpLst.length; j++) {
                    tmpLst[j].title = item.title;
                    retList.push(tmpLst[j]);
                }
            }
            neddNum = num - retList.length;
            if (neddNum == 0) {
                break;
            }
        }
    }
    return retList;
}

function getActivityType(key) {
    let nType = 0;
    if (!key || key.length == 0) {
        return nType;
    }
    let typeMap = {
        1: '合成活动',
        2: '置换活动',
        3: '分解活动',
        4: '兑换活动'
    };
    for (let k in typeMap) {
        if (typeMap[k].indexOf(key) != -1) {
            nType = parseInt(k);
            break;
        }
    }
    return nType;
}


function isObjectEmpty(obj) {
    return Object.keys(obj).length === 0;
}

async function getActivityList(activityType, noCache) {
    let key = `${WuBianConstKey.ActivityListData}#${activityType}`;
    let dataDict = magicJS.data.read(key, null);
    if (!dataDict || isObjectEmpty(dataDict) || noCache) {
        dataDict = await queryAllActivityList(activityType);
        magicJS.data.write(key, dataDict);
    }
    return dataDict;
}

async function queryAllActivityList(activityType, status = 0) {
    let dataDict = {};
    for (let i = 1; i < 3; i++) {
        let ret = await queryActivityList(activityType, i, status);
        if (ret && ret.code == 200) {
            magicJS.logger.info(`获取活动列表成功，当前页数：${i}`);
            const iData = ret.data;
            const itemList = iData.list;
            for (let item of itemList) {
                dataDict[item.hashId] = item;
            }
            if (iData.next == 0) {
                break;
            }
        }
    }
    return dataDict;
}

async function queryActivityList(activityType, page, status = 0) {
    const url = `https://api.wubian.pro/vmf/app/synthesis/activityList`;
    const reqData = {
        page: page,
        status: status,
    };
    if (activityType && activityType > 0) {
        reqData.activityType = String(activityType);
    }
    let options = {
        url: url,
        headers: gCommonHeaders,
        body: JSON.stringify(reqData),
    };
    let result = await magicJS.http.post(options).then(response => {
        try {
            let rspData = response.body;
            magicJS.logger.info(`rspData=${JSON.stringify(rspData)}`);
            return rspData;
        } catch (e) {
            magicJS.logger.error(e);
        }
    }).catch(err => {
        const msg = `请求活动列表数据异常\n${JSON.stringify(err)}`;
        magicJS.logger.error(msg);
    });
    return result;
}

async function doGetConvertGoodsList(activityId, noCache) {
    let key = `${WuBianConstKey.ActivityGoodsList}#${activityId}`;
    let data = magicJS.data.read(key, null);
    if (!data || noCache) {
        setConvertGoodsListReqTime(Date.now());
        let ret = await queryConvertGoodsList(activityId);
        if (ret && ret.code == 200) {
            data = ret.data;
            magicJS.data.write(key, data);
        }
    }
    return data;
}



async function queryConvertGoodsList(activityId) {
    const url = `https://api.wubian.pro/vmf/app/synthesis/convertGoodsList`;
    const reqData = {
        activityId: activityId,
    };
    let options = {
        url: url,
        headers: gCommonHeaders,
        body: JSON.stringify(reqData),
    };
    let result = await magicJS.http.post(options).then(response => {
        try {
            let rspData = response.body;
            magicJS.logger.info(`rspData=${JSON.stringify(rspData)}`);
            return rspData;
        } catch (e) {
            magicJS.logger.error(e);
        }
    }).catch(err => {
        const msg = `请求兑换消耗产品列表异常\n${JSON.stringify(err)}`;
        magicJS.logger.error(msg);
    });
    return result;
}


async function doGetCompositeGoodsList(activityId, noCache) {
    let key = `${WuBianConstKey.ActivityGoodsList}#${activityId}`;
    let data = magicJS.data.read(key, null);
    if (!data || noCache) {
        setCompositeGoodsListReqTime(Date.now());
        let ret = await queryCompositeGoodsList(activityId);
        if (ret && ret.code == 200) {
            data = ret.data;
            magicJS.data.write(key, data);
        }
    }
    return data;
}

async function queryCompositeGoodsList(activityId) {
    const url = `https://api.wubian.pro/vmf/app/synthesis/compositeGoodsList`;
    const reqData = {
        id: activityId,
    };
    let options = {
        url: url,
        headers: gCommonHeaders,
        body: JSON.stringify(reqData),
    };
    let result = await magicJS.http.post(options).then(response => {
        try {
            let rspData = response.body;
            magicJS.logger.info(`rspData=${JSON.stringify(rspData)}`);
            return rspData;
        } catch (e) {
            magicJS.logger.error(e);
        }
    }).catch(err => {
        const msg = `请求合成消耗产品列表异常\n${JSON.stringify(err)}`;
        magicJS.logger.error(msg);
    });
    return result;
}

async function doCreateConvertOrder(index, activityId, goodsList) {
    const url = `https://api.wubian.pro/vmf/app/synthesis/createConvertOrder`;
    const reqData = {
        activityId: activityId,
        goodsList: goodsList,
    };
    let options = {
        url: url,
        headers: gCommonHeaders,
        body: JSON.stringify(reqData),
    };
    let result = await magicJS.http.post(options).then(response => {
        try {
            let rspData = response.body;
            magicJS.logger.info(`rspData=${JSON.stringify(rspData)}`);
            gSynthesizeResultList.push(rspData);
            gSynthesizeResultDict[index] = { activityId: activityId, result: rspData, goodsList: goodsList };
            return rspData;
        } catch (e) {
            magicJS.logger.error(e);
        }
    }).catch(err => {
        const msg = `请求兑换异常\n${JSON.stringify(err)}`;
        magicJS.logger.error(msg);
    });
    return result;
}

async function doCreateCompositeOrder(index, activityId, goodsList) {
    const url = `https://api.wubian.pro/vmf/app/synthesis/createCompositeOrder`;
    const reqData = {
        activityId: activityId,
        goodsList: goodsList,
    };
    let options = {
        url: url,
        headers: gCommonHeaders,
        body: JSON.stringify(reqData),
    };
    let result = await magicJS.http.post(options).then(response => {
        try {
            let rspData = response.body;
            magicJS.logger.info(`rspData=${JSON.stringify(rspData)}`);
            gSynthesizeResultList.push(rspData);
            gSynthesizeResultDict[index] = { activityId: activityId, result: rspData, goodsList: goodsList };
            return rspData;
        } catch (e) {
            magicJS.logger.error(e);
        }
    }).catch(err => {
        const msg = `请求合成异常\n${JSON.stringify(err)}`;
        magicJS.logger.error(msg);
    });
    return result;
}

Main().catch((e) => magicJS.logger.log(`-\n ${e}`)).finally(() => magicJS.done());


//---SyncByPyScript---MagicJS3-start
function MagicJS(e = "MagicJS", t = "INFO") { const n = () => { const e = "undefined" != typeof $loon, t = "undefined" != typeof $task, r = "undefined" != typeof module, n = "undefined" != typeof $httpClient && !e, o = "undefined" != typeof $storm, i = "undefined" != typeof $environment && void 0 !== $environment["stash-build"]; var s = n || e || o || i; const a = "undefined" != typeof importModule; return { isLoon: e, isQuanX: t, isNode: r, isSurge: n, isStorm: o, isStash: i, isSurgeLike: s, isScriptable: a, get name() { return e ? "Loon" : t ? "QuantumultX" : r ? "NodeJS" : n ? "Surge" : a ? "Scriptable" : "unknown" }, get build() { return n ? $environment["surge-build"] : i ? $environment["stash-build"] : o ? $storm.buildVersion : void 0 }, get language() { if (n || i) return $environment.language }, get version() { return n ? $environment["surge-version"] : i ? $environment["stash-version"] : o ? $storm.appVersion : r ? process.version : void 0 }, get system() { return n ? $environment.system : r ? process.platform : void 0 }, get systemVersion() { if (o) return $storm.systemVersion }, get deviceName() { if (o) return $storm.deviceName } } }, o = (r, e = "INFO") => { let n = e, t = "\n"; const o = { SNIFFER: 6, DEBUG: 5, INFO: 4, NOTIFY: 3, WARNING: 2, ERROR: 1, CRITICAL: 0, NONE: -1 }, i = { SNIFFER: "", DEBUG: "", INFO: "", NOTIFY: "", WARNING: "❗ ", ERROR: "❌ ", CRITICAL: "❌ ", NONE: "" }, s = (e, t = "INFO") => { o[n] < o[t.toUpperCase()] || console.log(`██[${r}][${t}]` + i[t.toUpperCase()] + e + "\n") }; return { getLevel: () => n, setLevel: e => { n = e }, sniffer: (...e) => { e = e.join(t); s(e, "SNIFFER") }, log: (...e) => { e = e.join(t); console.log(`██[${r}]` + e + "\n") }, debug: (...e) => { e = e.join(t); s(e, "DEBUG") }, info: (...e) => { e = e.join(t); s(e, "INFO") }, notify: (...e) => { e = e.join(t); s(e, "NOTIFY") }, warning: (...e) => { e = e.join(t); s(e, "WARNING") }, error: (...e) => { e = e.join(t); s(e, "ERROR") }, retry: (...e) => { e = e.join(t); s(e, "RETRY") } } }; return new class { constructor(e, t) { var r; this._startTime = Date.now(), this.version = "3.0.0", this.scriptName = e, this.env = n(), this.logger = o(e, t), this.http = "function" == typeof MagicHttp ? MagicHttp(this.env, this.logger) : void 0, this.data = "function" == typeof MagicData ? MagicData(this.env, this.logger) : void 0, this.notification = "function" == typeof MagicNotification ? MagicNotification(this.scriptName, this.env, this.logger, this.http) : void 0, this.utils = "function" == typeof MagicUtils ? MagicUtils(this.env, this.logger) : void 0, this.qinglong = "function" == typeof MagicQingLong ? MagicQingLong(this.env, this.data, this.logger) : void 0, void 0 !== this.data && (t = this.data.read("magic_loglevel"), r = this.data.read("magic_bark_url"), t && this.logger.setLevel(t.toUpperCase()), r) && this.notification.setBark(r), this.logger.info(e + ", 开始执行!") } get isRequest() { return "undefined" != typeof $request } get isStrictRequest() { return "undefined" != typeof $request && "undefined" == typeof $response } get isResponse() { return "undefined" != typeof $response } get isDebug() { return "DEBUG" === this.logger.level } get request() { return "undefined" != typeof $request ? $request : void 0 } get response() { if ("undefined" != typeof $response) return $response.hasOwnProperty("status") && ($response.statusCode = $response.status), $response.hasOwnProperty("statusCode") && ($response.status = $response.statusCode), $response } log(...e) { this.logger.log(e) } toStr(e, t = null) { try { return JSON.stringify(e) } catch { return t } } toObj(e, t = null) { try { return JSON.parse(e) } catch { return t } } checkRecordRequestBody() { if (this.isRequest) { var t = $request.body; if (t) { var r = this.env, n = $request.path; let e = this.scriptName + "#" + n.replace("/", "_"); e = e.replace("?", "#"), r.isQuanX && $prefs.setValueForKey(t, e), (r.isLoon || r.isSurge) && $persistentStore.write(t, e), r.isNode && require("fs").writeFileSync(e + ".json", t, { flag: "w" }, e => console.log(e)) } } } getRequestBody() { var e = this.env, t = $request.path; let r = this.scriptName + "#" + t.replace("/", "_"); if (r = r.replace("?", "#"), e.isSurge || e.isLoon) return $persistentStore.read(r); if (e.isQuanX) return $prefs.valueForKey(r); if (e.isNode) { t = r + ".json", e = require("fs"); if (!e.existsSync(t)) return JSON.parse(e.readFileSync(t)) } } getResponseBody() { if ($response) return $response.body } parseCookies(e, t = !1) { let r = {}; return t ? e && e.split(";").forEach(function (e) { e = e.split("="); r[e.shift().trim()] = decodeURIComponent(e.join("=")) }) : e && e.split(";").forEach(function (e) { e = e.split("="); r[e.shift().trim()] = e.join("=") }), r } serializeCookies(e, t = !1) { var r = []; if (t) for (var n in e) { var o = e[n], n = n + "=" + encodeURIComponent(o); r.push(n) } else for (var i in e) { var s = e[i], i = i + "=" + s; r.push(i) } return r.join("; ") } parseSetCookies(e) { var t = e.split(/,\s*/).map(e => { var e = e.trim().split(/;\s*(?=[^=]+=[^;]*)/), t = e[0].split("=").map(e => e.trim()); const r = { name: t[0], value: t[1] }; return e.slice(1).forEach(e => { var [e, t] = e.split("=").map(e => e.trim()); r[e] = t || !0 }), r }), r = []; let n = 0; for (; n < t.length;) { var o, i = t[n]; r.push(i), i.Expires ? ((o = t[n + 1]) && (i.Expires = i.Expires + "," + o.name), n += 2) : n += 1 } return r } objToQueryStr(t, r) { let n = ""; for (const o in t) { let e = t[o]; null != e && "" !== e && ("object" == typeof e ? e = JSON.stringify(e) : r && (e = encodeURIComponent(e)), n += `${o}=${e}&`) } return n = n.substring(0, n.length - 1) } parseQueryStr(e) { var t = {}, r = (e = -1 < e.indexOf("?") ? e.split("?")[1] : e).split("&"); for (let e = 0; e < r.length; e++) { var n = r[e].split("="); t[n[0]] = n[1] } return t } deepClone(e, t) { for (var r in t = t || {}, e) "object" == typeof e[r] ? (t[r] = e[r].constructor === Array ? [] : {}, this.deepClone(e[r], t[r])) : t[r] = e[r]; return t } formatDate(e, t) { var r, n = { "M+": e.getMonth() + 1, "d+": e.getDate(), "H+": e.getHours(), "m+": e.getMinutes(), "s+": e.getSeconds(), "q+": Math.floor((e.getMonth() + 3) / 3), S: e.getMilliseconds() }; for (r in /(y+)/.test(t) && (t = t.replace(RegExp.$1, (e.getFullYear() + "").substr(4 - RegExp.$1.length))), n) new RegExp("(" + r + ")").test(t) && (t = t.replace(RegExp.$1, 1 == RegExp.$1.length ? n[r] : ("00" + n[r]).substr(("" + n[r]).length))); return t } parseDate(a, e) { let l = { y: 0, M: 1, d: 0, H: 0, h: 0, m: 0, s: 0, S: 0 }; (e = e || "yyyy-MM-dd").replace(/([^yMdHmsS]*?)(([yMdHmsS])\3*)([^yMdHmsS]*?)/g, function (e, t, r, n, o, i, s) { return a = a.replace(new RegExp(t + "(\\d{" + r.length + "})" + o), function (e, t) { return l[n] = parseInt(t), "" }), "" }), l.M--; e = new Date(l.y, l.M, l.d, l.H, l.m, l.s); return 0 !== l.S && e.setMilliseconds(l.S), e } getBaseDoneHeaders(e = {}) { return Object.assign({ "Access-Control-Allow-Origin": "*", "Access-Control-Allow-Methods": "POST,GET,OPTIONS,PUT,DELETE", "Access-Control-Allow-Headers": "Origin, X-Requested-With, Content-Type, Accept" }, e) } getHtmlDoneHeaders() { return this.getBaseDoneHeaders({ "Content-Type": "text/html;charset=UTF-8" }) } getJsonDoneHeaders() { return this.getBaseDoneHeaders({ "Content-Type": "text/json; charset=utf-8", Connection: "keep-alive" }) } doWxpusherSend(e) { var t = this.getJsonDoneHeaders(), t = (t.Host = "wxpusher.zjiecode.com", t["Content-Type"] = "application/json;charset=UTF-8", { url: "https://wxpusher.zjiecode.com/api/send/message", headers: t, body: JSON.stringify(e) }); return this.http.post(t) } fastWxpusherSend(e, t = "", r = "") { return this.doWxpusherSend({ appToken: "AT_7wDWqSoT8xpJCQqJtHpshKhw7kXc0XCW", content: e, summary: t, contentType: 1, topicIds: [], uids: ["UID_6P4B00X6Zv8U2oKC0I2R09emxtqq"], url: r, verifyPay: !1, verifyPayType: 0 }) } isEmpty(e) { return void 0 === e || null == e || "" == e || "null" == e || "undefined" == e || 0 === e.length } base64Encode(e) { var t, r, n, o = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"; let i, s, a; for (a = e.length, s = 0, i = ""; s < a;) { if (t = 255 & e.charCodeAt(s++), s == a) { i = (i += o.charAt(t >> 2)) + o.charAt((3 & t) << 4) + "=="; break } if (r = e.charCodeAt(s++), s == a) { i = (i = (i += o.charAt(t >> 2)) + o.charAt((3 & t) << 4 | (240 & r) >> 4)) + o.charAt((15 & r) << 2) + "="; break } n = e.charCodeAt(s++), i = (i = (i = (i += o.charAt(t >> 2)) + o.charAt((3 & t) << 4 | (240 & r) >> 4)) + o.charAt((15 & r) << 2 | (192 & n) >> 6)) + o.charAt(63 & n) } return i } base64Decode(e) { var t = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="; if (/([^\s]+[^0-9a-zA-Z\+\/\=]|[^0-9a-zA-Z\+\/\=]\s+)/.test(e)) throw new Error("Invalid base64 input"); var r, n, o, i, s, a, l = e.replace(/\s/g, ""); let u = "", c = 0; for (; c < l.length;)o = t.indexOf(l.charAt(c++)), r = (15 & (i = t.indexOf(l.charAt(c++)))) << 4 | (s = t.indexOf(l.charAt(c++))) >> 2, n = (3 & s) << 6 | (a = t.indexOf(l.charAt(c++))), u += String.fromCharCode(o << 2 | i >> 4), 64 !== s && (u += String.fromCharCode(r)), 64 !== a && (u += String.fromCharCode(n)); return u = this.utf8Decode(u) } utf8Decode(t) { let e = [], r = 0, n = 0, o = 0; for (t = t.replace(/\r\n/g, "\n"); r < t.length;) { n = 255 & t.charCodeAt(r), o = 0, o = n <= 191 ? (n &= 127, 1) : n <= 223 ? (n &= 31, 2) : n <= 239 ? (n &= 15, 3) : (n &= 7, 4); for (let e = 1; e < o; ++e)n = n << 6 | 63 & t.charCodeAt(e + r); 4 === o ? (n -= 65536, e.push(String.fromCharCode(55296 | n >> 10 & 1023)), e.push(String.fromCharCode(56320 | 1023 & n))) : e.push(String.fromCharCode(n)), r += o } return e.join("") } parseJwt(e) { try { var t = e.split("."), r = t[0].replace(/-/g, "+").replace(/_/g, "/"), n = this.base64Decode(r).replace(/\0/g, ""), o = JSON.parse(n), i = t[1].replace(/-/g, "+").replace(/_/g, "/"), s = this.base64Decode(i).replace(/\0/g, ""); return { header: o, payload: JSON.parse(s), signature: t[2] } } catch (e) { return this.log(e), null } } costTime() { var e = this.scriptName + "执行完毕！", t = (this._endTime = (new Date).getTime(), this._endTime - this._startTime); this.logger.info(e + `耗时【${t / 1e3}】秒`) } done = (e = {}) => { this.costTime(), "undefined" != typeof $done && $done(e) } }(e, t) } function MagicHttp(d, p) { var e; let g; d.isNode && (e = require("axios"), g = e.create()); class t { constructor(e = !0) { this.handlers = [], this.isRequest = e } use(e, t, r) { return "function" == typeof e && p.debug("Register fulfilled " + e.name), "function" == typeof t && p.debug("Register rejected " + t.name), this.handlers.push({ fulfilled: e, rejected: t, synchronous: !(!r || "boolean" != typeof r.synchronous) && r.synchronous, runWhen: r ? r.runWhen : null }), this.handlers.length - 1 } eject(e) { this.handlers[e] && (this.handlers[e] = null) } forEach(t) { this.handlers.forEach(e => { null !== e && t(e) }) } } function f(e) { let r = { ...e }; return r.params && !d.isNode && (e = Object.keys(r.params).map(e => { var t = encodeURIComponent(e); return r.url = r.url.replace(new RegExp(e + "=[^&]*", "ig"), ""), r.url = r.url.replace(new RegExp(t + "=[^&]*", "ig"), ""), t + "=" + encodeURIComponent(r.params[e]) }).join("&"), r.url.indexOf("?") < 0 && (r.url += "?"), /(&|\?)$/g.test(r.url) || (r.url += "&"), r.url += e, delete r.params, p.debug("Params to QueryString: " + r.url)), r } const h = (e, t = null) => { if (e) { t = { ...e, config: e.config || t, status: e.statusCode || e.status, body: e.body || e.data, headers: e.headers || e.header }; if ("string" == typeof t.body) try { t.body = JSON.parse(t.body) } catch { } return delete t.data, t } return e }; const y = (e, t = null) => { if (e && 400 <= e.status) return p.debug("Raise exception when status code is " + e.status), { name: "RequestException", message: "Request failed with status code " + e.status, config: t || e.config, response: e } }, m = { request: new t, response: new t(!1) }; let v = [], b = [], S = !0; function N(e) { return e = f(e), p.debug(`HTTP ${e.method.toUpperCase()}:` + "\n" + JSON.stringify(e)), e } function E(t) { try { t = t && h(t), p.sniffer(`HTTP ${t.config.method.toUpperCase()}:` + "\n" + JSON.stringify(t.config) + "\nSTATUS CODE:\n" + t.status + "\nRESPONSE:\n" + ("object" == typeof t.body ? JSON.stringify(t.body) : t.body)); var e = y(t); return e ? Promise.reject(e) : t } catch (e) { return p.error(e), t } } const r = (e, r) => { let n; r = ((e, t) => { let r = "object" == typeof t ? { headers: {}, ...t } : { url: t, headers: {} }; return r.method || (r.method = e), !0 === (r = f(r)).rewrite && (d.isSurge ? (r.headers["X-Surge-Skip-Scripting"] = !1, delete r.rewrite) : d.isQuanX && (r.hints = !1, delete r.rewrite)), d.isSurgeLike ? (t = r.headers["content-type"] || r.headers["Content-Type"], "GET" !== r.method && t && 0 <= t.indexOf("application/json") && r.body instanceof Array && (r.body = JSON.stringify(r.body), p.debug("Convert Array object to String: " + r.body))) : d.isQuanX ? (r.hasOwnProperty("body") && "string" != typeof r.body && (r.body = JSON.stringify(r.body)), r.method = e) : d.isNode && ("POST" === e || "PUT" === e || "PATCH" === e || "DELETE" === e ? r.data = r.data || r.body : "GET" === e && (r.params = r.params || r.body), delete r.body), r })(e.toUpperCase(), r), n = d.isNode ? g : d.isSurgeLike ? i => new Promise((n, o) => { $httpClient[e.toLowerCase()](i, (e, t, r) => { e ? (e = { name: e.name || e, message: e.message || e, stack: e.stack || e, config: i, response: h(t) }, o(e)) : (t.config = i, t.body = r, n(t)) }) }) : n => new Promise((r, t) => { $task.fetch(n).then(e => { e = h(e, n); var t = y(e, n); if (t) return Promise.reject(t); r(e) }).catch(e => { e = { name: e.message || e.error, message: e.message || e.error, stack: e.error, config: n, response: e.response ? h(e.response) : null }; t(e) }) }); let o; var t = r; try { v = [], b = [], m.request.forEach(e => { "function" == typeof e.runWhen && !1 === e.runWhen(t) || (S = S && e.synchronous, v.unshift(e.fulfilled, e.rejected)) }), m.response.forEach(e => { b.push(e.fulfilled, e.rejected) }) } catch (e) { p.error(`Failed to register interceptors: ${e}.`) } var i = [N, void 0], s = [E, void 0]; if (S) { for (p.debug("Interceptors are executed in synchronous mode"), Array.prototype.unshift.apply(v, i), v = v.concat([N, void 0]); v.length;) { var a = v.shift(), l = v.shift(); try { "function" == typeof a && p.debug("Executing request fulfilled " + a.name), r = a(r) } catch (e) { "function" == typeof l && p.debug("Executing request rejected " + l.name), l(e); break } } try { o = (!d.isNode && r.timeout ? c : n)(r) } catch (e) { return Promise.reject(e) } for (Array.prototype.unshift.apply(b, s); b.length;)o = o.then(b.shift(), b.shift()); return o } { p.debug("Interceptors are executed in asynchronous mode"); let t = [n, void 0]; for (Array.prototype.unshift.apply(t, i), Array.prototype.unshift.apply(t, v), t = (t = t.concat(s)).concat(b), o = Promise.resolve(r); t.length;)try { let e = t.shift(); var u = t.shift(); "function" == typeof (e = !d.isNode && r.timeout && e === n ? c : e) && p.debug("Executing request fulfilled " + e.name), "function" == typeof u && p.debug("Executing request rejected " + u.name), o = o.then(e, u) } catch (e) { p.error("request exception: " + e) } return o } function c(r) { try { var e = new Promise((e, t) => { setTimeout(() => { var e = { message: `timeout of ${r.timeout}ms exceeded.`, config: r }; t(e) }, r.timeout) }); return Promise.race([n(r), e]) } catch (e) { p.error(`Request Timeout exception: ${e}.`) } } }; return { request: r, interceptors: m, convertHeadersToLowerCase: r => Object.keys(r).reduce((e, t) => (e[t.toLowerCase()] = r[t], e), {}), convertHeadersToCamelCase: r => Object.keys(r).reduce((e, t) => { return e[t.split("-").map(e => e[0].toUpperCase() + e.slice(1)).join("-")] = r[t], e }, {}), modifyResponse: h, get: e => r("GET", e), post: e => r("POST", e), put: e => r("PUT", e), patch: e => r("PATCH", e), delete: e => r("DELETE", e), head: e => r("HEAD", e), options: e => r("OPTIONS", e) } } function MagicData(d, p) { let g = { fs: void 0, data: {} }; if (d.isNode) { g.fs = require("fs"); try { g.fs.accessSync("./magic.json", g.fs.constants.R_OK | g.fs.constants.W_OK) } catch (e) { g.fs.writeFileSync("./magic.json", "{}", { encoding: "utf8" }) } g.data = require("./magic.json") } const s = (e, t) => "object" != typeof t && e === t, a = e => "true" === e || "false" !== e && (void 0 === e ? null : e), l = (e, t, r, n) => { if (r) try { e = !0 === (e = "string" == typeof e ? JSON.parse(e) : e).magic_session ? e[r] : null } catch { e = null } if ("string" == typeof e && "null" !== e) try { e = JSON.parse(e) } catch { } return null == (e = !1 === n && e && !0 === e.magic_session ? null : e) && null != t && (e = t), e = a(e) }, f = t => { if ("string" != typeof t) return t instanceof Array || null == t || t != t || "boolean" == typeof t ? {} : t; { let e = {}; try { var r = typeof (e = JSON.parse(t)); ("object" != r || e instanceof Array || "bool" == r || null === e) && (e = {}) } catch { } return e } }, u = (e, t = null, r = "", n = !1, o = null) => { let i = ""; return i = o || d.isNode ? ((e, t = null, r = "", n = !1, o = null) => { o = o || g.data; return val = o && void 0 !== o[e] && null !== o[e] ? o[e] : r ? {} : null, val = l(val, t, r, n) })(e, t, r, n, o) : (d.isSurgeLike ? i = $persistentStore.read(e) : d.isQuanX && (i = $prefs.valueForKey(e)), l(i, t, r, n)), p.debug(`READ DATA [${e}]${r ? `[${r}]` : ""} <${typeof i}>` + "\n" + JSON.stringify(i)), i }, c = (e, t, r = "", n = null) => { if (void 0 === t || t != t) return !1; d.isNode || "boolean" != typeof t && "number" != typeof t || (t = String(t)); let o = ""; var i, s, a, l, u, c; if (n || d.isNode ? o = ([i, s, a = "", l = null] = [e, t, r, n], c = l || g.data, c = f(c), a ? ((u = f(c[i])).magic_session = !0, u[a] = s, c[i] = u) : c[i] = s, null !== l && (l = c), c) : r ? (d.isSurgeLike ? o = $persistentStore.read(e) ? $persistentStore.read(e) : o : d.isQuanX && (o = $prefs.valueForKey(e) ? $prefs.valueForKey(e) : o), (o = f(o)).magic_session = !0, o[r] = t) : o = t, o && "object" == typeof o && (o = JSON.stringify(o, null, 4)), p.debug(`WRITE DATA [${e}]${r ? `[${r}]` : ""} <${typeof t}>` + "\n" + JSON.stringify(t)), !n) { if (d.isSurgeLike) return $persistentStore.write(o, e); if (d.isQuanX) return $prefs.setValueForKey(o, e); if (d.isNode) try { g.fs.writeFileSync("./magic.json", o) } catch (e) { return p.error(e), !1 } } return !0 }; return { read: u, write: c, del: (e, t = "", r = null) => { let n = {}; if (r || d.isNode) n = (o = e, i = t, s = r || g.data, s = f(s), i ? (delete (obj = f(s[o]))[i], s[o] = obj) : delete s[o], s), r ? r = n : g.fs.writeFileSync("./magic.json", JSON.stringify(n, null, 4)); else if (t) { d.isSurgeLike ? n = $persistentStore.read(e) : d.isQuanX && (n = $prefs.valueForKey(e)), delete (n = f(n))[t]; i = JSON.stringify(n, null, 4); c(e, i) } else { if (d.isStorm) return $persistentStore.remove(e); if (d.isSurgeLike) return $persistentStore.write(null, e); if (d.isQuanX) return $prefs.removeValueForKey(e) } var o, i, s; p.debug(`DELETE KEY [${e}]` + (t ? `[${t}]` : "")) }, update: (e, t, r, n = s, o = null) => { var i; return t = a(t), !0 !== n(u(e, null, r, !1, o), t) && (i = c(e, t, r, o), e = u(e, null, r, !1, o), n === s && "object" == typeof e ? i : n(t, e)) }, allSessions: (e, t = null) => { let r = {}; t = u(e, null, null, !0, t); return !0 === (t = f(t)).magic_session && delete (r = { ...t }).magic_session, p.debug(`READ ALL SESSIONS [${e}] <${typeof r}>` + "\n" + JSON.stringify(r, null, 4)), r }, allSessionNames: (e, t = null) => { let r = []; t = u(e, null, null, !0, t), t = f(t); return r = !0 !== t.magic_session ? [] : Object.keys(t).filter(e => "magic_session" !== e), p.debug(`READ ALL SESSIONS [${e}] <${typeof r}>` + "\n" + JSON.stringify(r, null, 4)), r }, defaultValueComparator: s, convertToObject: f } } function MagicNotification(i, o, s, a) { let l = null, u = null, c = []; function d(e = i, t = "", r = "", n = "") { n = (t => { try { let e = {}; var r; return "string" == typeof t ? 0 < t.length && (o.isLoon ? e = { openUrl: t } : o.isQuanX ? e = { "open-url": t } : o.isSurge && (e = { url: t })) : "object" == typeof t && (o.isLoon ? (e.openUrl = t["open-url"] || "", e.mediaUrl = t["media-url"] || "") : o.isQuanX ? e = t["open-url"] || t["media-url"] ? t : {} : o.isSurge && (r = t["open-url"] || t.openUrl, e = r ? { url: r } : {})), e } catch (e) { s.error("通知选项转换失败" + e) } return t })(n), 1 === arguments.length && (e = i, t = "", r = arguments[0]), s.notify("\ntitle:" + e + "\nsubTitle:" + t + "\nbody:" + r + "\noptions:" + ("object" == typeof n ? JSON.stringify(n) : n)), o.isSurge ? $notification.post(e, t, r, n) : o.isLoon ? n ? $notification.post(e, t, r, n) : $notification.post(e, t, r) : o.isQuanX && $notify(e, t, r, n), l && u && p(e, t, r) } function p(e = i, t = "", r = "", n) { if (void 0 === a || void 0 === a.post) throw "Bark notification needs to import MagicHttp module."; e = { url: l, headers: { "content-type": "application/json; charset=utf-8" }, body: { title: e, body: t ? t + "\n" + r : r, device_key: u } }; a.post(e).catch(e => { s.error("Bark notify error: " + e) }) } return { post: d, debug: function (e = i, t = "", r = "", n = "") { "DEBUG" === s.getLevel() && (1 === arguments.length && (e = i, t = "", r = arguments[0]), this.post(e, t, r, n)) }, bark: p, setBark: e => { try { var t = e.replace(/\/+$/g, ""); l = /^https?:\/\/([^/]*)/.exec(t)[0] + "/push", u = /\/([^\/]+)\/?$/.exec(t)[1] } catch (e) { s.error(`Bark url error: ${e}.`) } }, appendNotifyInfo: function (e, t) { 1 == t ? c = e : c.push(e) }, prependNotifyInfo: function (e) { c.splice(0, 0, e) }, msg: function (e, t, r, n) { var o = {}; r && (o["open-url"] = r), n && (o["media-url"] = n), (t = t && 0 != t.length ? t : Array.isArray(c) ? c.join("\n") : c) && 0 < t.length && d(i, "", t, o) } } } function MagicUtils(n, u) { const e = (e, t = "yyyy-MM-dd hh:mm:ss") => { var r, n = { "M+": e.getMonth() + 1, "d+": e.getDate(), "h+": e.getHours(), "m+": e.getMinutes(), "s+": e.getSeconds(), "q+": Math.floor((e.getMonth() + 3) / 3), S: e.getMilliseconds() }; for (r in /(y+)/.test(t) && (t = t.replace(RegExp.$1, (e.getFullYear() + "").substr(4 - RegExp.$1.length))), n) new RegExp("(" + r + ")").test(t) && (t = t.replace(RegExp.$1, 1 === RegExp.$1.length ? n[r] : ("00" + n[r]).substr(("" + n[r]).length))); return t }; return { retry: (i, s = 5, a = 0, l = null) => (...e) => new Promise((r, n) => { function o(...t) { Promise.resolve().then(() => i.apply(this, t)).then(e => { "function" == typeof l ? Promise.resolve().then(() => l(e)).then(() => { r(e) }).catch(e => { 1 <= s ? 0 < a ? setTimeout(() => o.apply(this, t), a) : o.apply(this, t) : n(e), s-- }) : r(e) }).catch(e => { u.error(e), 1 <= s && 0 < a ? setTimeout(() => o.apply(this, t), a) : 1 <= s ? o.apply(this, t) : n(e), s-- }) } o.apply(this, e) }), formatTime: e, now: () => e(new Date, "yyyy-MM-dd hh:mm:ss"), today: () => e(new Date, "yyyy-MM-dd"), sleep: t => new Promise(e => setTimeout(e, t)), assert: (e, t = null) => { var r; n.isNode ? (r = require("assert"), t ? r(e, t) : r(e)) : !0 !== e && u.error("AssertionError: " + (t || "The expression evaluated to a falsy value.")) } } } function MagicQingLong(e, a, o) { let i = "", s = "", l = "", u = "", c = "", t = ""; const d = "magic.json", p = MagicHttp(e, o); async function r() { return l = l || a.read("magic_qlclient"), u = u || a.read("magic_qlsecrt"), s = s || a.read("magic_qlname"), c = c || a.read("magic_qlpwd"), i && l && u ? (o.info("Get token from QingLong Panel"), await p.get({ url: "/open/auth/token", headers: { "content-type": "application/json" }, params: { client_id: l, client_secret: u } }).then(e => { if (!(0 < Object.keys(e.body).length && e.body.data && e.body.data.token)) throw new Error("Get QingLong Panel token failed."); o.info("Successfully logged in to QingLong Panel"), t = e.body.data.token, a.write("magic_qltoken", t) }).catch(e => { o.error("Error logging in to QingLong Panel.\n" + (e.message || e)) })) : i && s && c && await p.post({ url: "/api/user/login", headers: { "content-type": "application/json" }, body: { username: s, password: c } }).then(e => { o.info("Successfully logged in to QingLong Panel"), t = e.body.data.token, a.write("magic_qltoken", t) }).catch(e => { o.error("Error logging in to QingLong Panel.\n" + (e.message || e)) }), t } async function g(e) { let t = []; return await p.post({ url: "/api/envs", headers: { "content-type": "application/json" }, body: e }).then(e => { 200 === e.body.code ? e.body.data.forEach(e => { o.debug(`QINGLONG ADD ENV ${e.name} <${typeof e.value}> (${e.id})` + "\n" + JSON.stringify(e)), t.push(e.id) }) : o.error("Error adding environments variable from QingLong Panel.\n" + JSON.stringify(e)) }).catch(e => { o.error("Error adding environments variable from QingLong Panel.\n" + (e.message || e)) }), t } async function n(n = null, e = "", t) { let o = []; return await p.get({ url: "/api/envs", headers: { "content-type": "application/json" }, params: { searchValue: e } }).then(e => { if (200 !== e.body.code) throw new Error("Error reading environment variable from QingLong Panel.\n" + JSON.stringify(e)); e = e.body.data; if (n) { var t = []; for (const r of e) r.name === n && o.push(r); o = t } o = e }).catch(e => { throw new Error("Error reading environments variable from QingLong Panel.\n" + (e.message || e)) }), o } async function f(e, t = "") { let r = ""; return await p.get({ url: "/api/scripts/" + e, params: { path: t } }).then(e => { if (200 !== e.body.code) throw new Error("Error reading data from QingLong Panel.\n" + JSON.stringify(e)); r = e.body.data }).catch(e => { throw new Error("Error reading data from QingLong Panel.\n" + (e.message || e)) }), r } async function h(e, t = "", r = "") { let n = !1; return await p.put({ url: "/api/scripts", headers: { "content-type": "application/json" }, body: { filename: e, path: t, content: r } }).then(e => { 200 === e.body.code ? n = !0 : o.error("Error reading data from QingLong Panel.\n" + JSON.stringify(e)) }).catch(e => { o.error("Error reading data from QingLong Panel.\n" + (e.message || e)) }), n } return p.interceptors.request.use(function (e) { return i = i || a.read("magic_qlurl"), e.url.indexOf(i) < 0 && (e.url = "" + i + e.url), { ...e, timeout: 3e3 } }, void 0), p.interceptors.request.use(function (e) { return (l = l || a.read("magic_qlclient")) && (e.url = e.url.replace("/api/", "/open/")), e }, void 0, { runWhen: e => e.url.indexOf("api/user/login") < 0 && e.url.indexOf("open/auth/token") < 0 }), p.interceptors.request.use(async function (e) { return (t = t || a.read("magic_qltoken", "")) || await r(), e.headers.authorization = "Bearer " + t, e }, void 0, { runWhen: e => e.url.indexOf("api/user/login") < 0 && e.url.indexOf("open/auth/token") < 0 }), p.interceptors.request.use(function (e) { return e.params = { ...e.params, t: Date.now() }, e }, void 0, { runWhen: e => e.url.indexOf("open/auth/token") < 0 }), p.interceptors.request.use(function (e) { return i = i || a.read("magic_qlurl"), t = t || a.read("magic_qltoken"), o.debug("QingLong url: " + i + "\nQingLong token: " + t), e }, void 0), p.interceptors.response.use(void 0, async function (e) { try { var t = e.message || e.error || JSON.stringify(e); return (0 <= t.indexOf("NSURLErrorDomain") && 0 <= t.indexOf("-1012") || e.response && 401 === e.response.status) && e.config && !0 !== e.config.refreshToken ? (o.warning("QingLong Panel token has expired"), o.info("Refreshing the QingLong Panel token"), await r(), e.config.refreshToken = !0, o.info("Call the previous method again"), await p.request(e.config.method, e.config)) : Promise.reject(e) } catch (e) { return Promise.reject(e) } }), { url: i || a.read("magic_qlurl"), init: (e, t, r, n, o) => { i = e, l = t, u = r, s = n, c = o }, getToken: r, setEnv: async function (t, r, n = null) { if (i = i || a.read("magic_qlurl"), null === n) { var e = await g([{ name: t, value: r }]); if (e && 1 === e.length) return e[0] } else await p.put({ url: "/api/envs", headers: { "content-type": "application/json" }, body: { name: t, value: r, id: n } }).then(e => { if (200 === e.body.code) return o.debug(`QINGLONG UPDATE ENV ${t} <${typeof r}> (${n})` + "\n" + JSON.stringify(r)), !0; o.error("Error adding environment variable from QingLong Panel.\n" + JSON.stringify(e)) }).catch(e => (o.error("Error adding environment variable from QingLong Panel.\n" + (e.message || e)), !1)) }, setEnvs: g, getEnv: async function (e) { let t = null; for (const r of await n()) if (r.id === e) { t = r; break } return t }, getEnvs: n, delEnvs: async function (t) { return p.delete({ url: "/api/envs", headers: { accept: "application/json", "accept-language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6", connection: "keep-alive", "content-type": "application/json;charset=UTF-8", "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.5005.63 Safari/537.36 Edg/102.0.1245.30" }, body: t }).then(e => 200 === e.body.code ? (o.debug("QINGLONG DELETE ENV IDS: " + t), !0) : (o.error("Error deleting environments variable from QingLong Panel.\n" + JSON.stringify(e)), !1)).catch(e => { o.error("Error deleting environments variable from QingLong Panel.\n" + (e.message || e)) }) }, disableEnvs: async function (t) { let r = !1; return await p.put({ url: "/api/envs/disable", headers: { accept: "application/json", "accept-Language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6", connection: "keep-alive", "content-type": "application/json;charset=UTF-8", "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.5005.63 Safari/537.36 Edg/102.0.1245.30" }, body: t }).then(e => { 200 === e.body.code ? (o.debug("QINGLONG DISABLED ENV IDS: " + t), r = !0) : o.error("Error disabling environments variable from QingLong Panel.\n" + JSON.stringify(e)) }).catch(e => { o.error("Error disabling environments variable from QingLong Panel.\n" + (e.message || e)) }), r }, enableEnvs: async function (t) { let r = !1; return await p.put({ url: "/api/envs/enable", headers: { accept: "application/json", "accept-language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6", connection: "keep-alive", "content-type": "application/json;charset=UTF-8", "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.5005.63 Safari/537.36 Edg/102.0.1245.30" }, body: t }).then(e => { 200 === e.body.code ? (o.debug("QINGLONG ENABLED ENV IDS: " + t), r = !0) : o.error("Error enabling environments variable from Qilong panel.\n" + JSON.stringify(e)) }).catch(e => { o.error("Error enabling environments variable from Qilong panel.\n" + (e.message || e)) }), r }, addScript: async function (e, t = "", r = "") { let n = !1; return await p.post({ url: "/api/scripts", headers: { "content-type": "application/json" }, body: { filename: e, path: t, content: r } }).then(e => { 200 === e.body.code ? n = !0 : o.error("Error reading data from QingLong Panel.\n" + JSON.stringify(e)) }).catch(e => { o.error("Error reading data from QingLong Panel.\n" + (e.message || e)) }), n }, getScript: f, editScript: h, delScript: async function (e, t = "") { let r = !1; return await p.delete({ url: "/api/scripts", headers: { "content-type": "application/json" }, body: { filename: e, path: t } }).then(e => { 200 === e.body.code ? r = !0 : o.error("Error reading data from QingLong Panel.\n" + JSON.stringify(e)) }).catch(e => { o.error("Error reading data from QingLong Panel.\n" + (e.message || e)) }), r }, write: async function (e, t, r = "") { var n = await f(d, ""), o = a.convertToObject(n), e = a.write(e, t, r, o), n = JSON.stringify(o, null, 4); return await h(d, "", n) && e }, read: async function (e, t, r = "", n = !1) { var o = await f(d, ""), o = a.convertToObject(o); return a.read(e, t, r, n, o) }, del: async function (e, t = "") { var r = await f(d, ""), n = a.convertToObject(r), e = a.del(e, t, n), r = JSON.stringify(n, null, 4), t = await h(d, "", r); return e && t }, update: async function (e, t, r, n = a.defaultValueComparator) { var o = await f(d, ""), i = a.convertToObject(o), e = a.update(e, t, r, n, i); let s = !1; return !0 === e && (o = JSON.stringify(i, null, 4), s = await h(d, "", o)), e && s }, batchWrite: async function (...e) { var t, r = await f(d, ""), n = a.convertToObject(r); for (t of e) a.write(t[0], t[1], void 0 !== t[2] ? t[2] : "", n); return r = JSON.stringify(n, null, 4), h(d, "", r) }, batchRead: async function (...e) { var t, r = await f(d, ""), n = a.convertToObject(r), o = []; for (t of e) { var i = a.read(t[0], t[1], void 0 !== t[2] ? t[2] : "", "boolean" == typeof t[3] && t[3], n); o.push(i) } return o }, batchUpdate: async function (...e) { var t, r = await f(d, ""), n = a.convertToObject(r); for (t of e) a.update(t[0], t[1], void 0 !== t[2] ? t[2] : "", void 0 !== t[3] ? t.comparator : a.defaultValueComparator, n); return r = JSON.stringify(n, null, 4), h(d, "", r) }, batchDel: async function (...e) { var t, r = await f(d, ""), n = a.convertToObject(r); for (t of e) a.del(t[0], void 0 !== t[1] ? t[1] : "", n); return r = JSON.stringify(n, null, 4), h(d, "", r) }, allSessions: async function (e) { var t = await f(d, ""), t = a.convertToObject(t); return a.allSessions(e, t) }, allSessionNames: async function (e) { var t = await f(d, ""), t = a.convertToObject(t); return a.allSessionNames(e, t) } } }
//---SyncByPyScript---MagicJS3-end